//
//  FailedBanksCoreDataEnv.m
//  FailedBanks
//
//  Created by Timothy Shiu on 7/13/12.
//  Copyright (c) 2012 Shiu. All rights reserved.
//

#import "FailedBanksCoreDataEnv.h"


@interface FailedBanksCoreDataEnv ()

@property (strong,nonatomic) NSString* directory;
@property (strong,nonatomic) UIManagedDocument* document;
@property (strong,nonatomic) NSNotificationCenter* center;

@end

@implementation FailedBanksCoreDataEnv

@synthesize context = _context;
@synthesize model = _model;
@synthesize coordinator = _coordinator;
@synthesize fetcher = _fetcher;

@synthesize document = _document;
@synthesize directory = _directory;
@synthesize center = _center;

- (NSNotificationCenter*) center
{
    if (_center != nil) { return _center; }
    _center = [NSNotificationCenter defaultCenter];
    return _center;
}

- (void) openCoreData
{
    NSNotificationCenter* center = self.center;
    NSManagedObjectContext* context = self.document.managedObjectContext;
    [center addObserver:self selector:@selector(contextChanged:) name:NSManagedObjectContextObjectsDidChangeNotification object:context];
}

- (void) closeCoreData
{
    NSNotificationCenter* center = self.center;
    NSManagedObjectContext* context = self.document.managedObjectContext;    
    [center removeObserver:self name:NSManagedObjectContextObjectsDidChangeNotification object:context];
}

- (void) contextChanged: (NSNotification*) notification
{
    
}

- (NSManagedObjectModel*) model
{ // schema
    if (_model != nil) { return _model; }
    NSBundle * bundle = [NSBundle mainBundle];
    NSURL *url = [bundle URLForResource:@"FailedBanks" withExtension:@"momd"];
    _model = [[NSManagedObjectModel alloc] initWithContentsOfURL:url];
    return _model;
}

- (NSManagedObjectContext*) context
{
    if (_context != nil) { return _context; }
    NSPersistentStoreCoordinator* coordinator = self.coordinator;
    if (coordinator == nil) { return _context; }
    _context = [[NSManagedObjectContext alloc] init];
    _context.persistentStoreCoordinator = coordinator;
    return _context;
}

- (NSPersistentStoreCoordinator*) coordinator
{
    if (_coordinator != nil) { return _coordinator; }
    
    NSFileManager* fs = [NSFileManager defaultManager];
    NSArray* ls = [fs URLsForDirectory:NSDocumentDirectory inDomains:NSUserDomainMask];    
    NSURL *url = [[ls lastObject] URLByAppendingPathComponent:@"hello.sqlite3"];
    NSError *error = nil;
    _coordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:self.model];
    NSPersistentStore * store = nil;
    store = [_coordinator addPersistentStoreWithType:NSSQLiteStoreType 
                                       configuration:nil URL:url options:nil error:&error];
    if (store) {
        
    }
    return _coordinator;
}

- (NSFetchedResultsController*) fetcher
{
    if (_fetcher != nil) { return _fetcher; }
    NSString* entity = @"FailedBanksCoreData";
    NSString* ident = @"ident";
    NSFetchRequest* request = [NSFetchRequest fetchRequestWithEntityName:entity];
    NSSortDescriptor* sorter = [NSSortDescriptor sortDescriptorWithKey:ident ascending:YES];
    request.sortDescriptors = [NSArray arrayWithObject:sorter];
    request.predicate = nil;
    _fetcher = [[NSFetchedResultsController alloc] initWithFetchRequest:request 
                                                   managedObjectContext:self.context 
                                                     sectionNameKeyPath:nil 
                                                              cacheName:nil];
    NSError* error = nil;
    [_fetcher performFetch:&error];
    return _fetcher;
}

- (id) init
{
    self = [super init];
    if (self == nil) { return self; }
    BOOL ok = NO;
    ok = self.model != nil;
    ok = self.context != nil;
    ok = self.coordinator != nil;
    ok = self.fetcher != nil;
    if (!ok) {
        
    }
    [self openCoreData];

    return self;
}
- (UIManagedDocument*) document
{
    if (_document != nil) { return _document; }
    // NSURL* url = [NSURL URLWithString:@""]; 
    // _document = [[UIManagedDocument] initWithFileURL:url];
    // [[UIManagedDocument alloc] new]
    return _document;
}

- (void) documentIsReady
{
    UIDocumentState state = self.document.documentState;
    if (state == UIDocumentStateNormal) {
       //  NSManagedObjectContext* context = self.document.managedObjectContext;
    }
}

- (void) openDocument:(UIManagedDocument*) document
{
    [document 
     openWithCompletionHandler:^(BOOL success) 
     {
         if (success) { 
             [self documentIsReady]; 
         } else { 
             NSLog(@"failed to open document %@", document.fileURL); 
         }
     }
     ];
}

- (void) closeDocument:(UIManagedDocument*) document
{
    [document
     closeWithCompletionHandler:^(BOOL success) 
     {
         if (!success) {
             NSLog(@"failed to close document %@", document.fileURL);
         }
     } 
     ];    
}

- (void) saveDocument:(UIManagedDocument*) document
{
    [document
     saveToURL:document.fileURL forSaveOperation:UIDocumentSaveForOverwriting 
     completionHandler:^(BOOL success) {
         if (success) { 
             [self documentIsReady]; 
         } else { 
             NSLog(@"failed to save document %@", document.fileURL); 
         }
     }];
}

- (void) createDocument:(UIManagedDocument*) document
{
    [document 
     saveToURL:document.fileURL forSaveOperation:UIDocumentSaveForCreating 
     completionHandler:^(BOOL success) 
     {
         if (success) { [self documentIsReady]; } 
         else { NSLog(@"failed to create document %@", document.fileURL); }
     }
     ];
}


@end
