
//
//  FailedBanksDatabase.m
//  FailedBanks
//
//  Created by Timothy Shiu on 6/28/12.
//  Copyright (c) 2012 Shiu. All rights reserved.
//

#import "FailedBanksDatabase.h"
#import <sqlite3.h>

@interface FailedBanksDatabase ()

@property (readonly) sqlite3 * sqliteDatabase;
@property (readonly) const char* sqlitePath;
@end

@implementation FailedBanksDatabase

@synthesize sqliteDatabase = _sqliteDatabase;
@synthesize sqlitePath = _sqlitePath;
@synthesize array = _array;

- (const char *) sqlitePath
{
    if (_sqlitePath) { return _sqlitePath; }
    NSBundle *bundle = [NSBundle mainBundle];
    NSString* file;
    file = [bundle pathForResource:@"banklist" 
                            ofType:@"sqlite3"];
    _sqlitePath = [file UTF8String];
    return _sqlitePath;
}

- (sqlite3_stmt*) makeQuery:(NSString*) sql
{
    const char* query = [sql UTF8String];
    sqlite3_stmt* stmt = nil;
    sqlite3* database = [self sqliteDatabase];
    int status = sqlite3_prepare_v2(database, query, -1, &stmt, nil);
    if (status == SQLITE_OK) { 
        NSLog(@"%@\nsuceeded", sql);
        return stmt;
    } else {
        NSLog(@"%@\nfailed", sql);
        return nil; 
    } 
}

- (void) initArray
{
    NSString* query = @"select Ident, Bank_Name, City, State from banklist";
    sqlite3_stmt* stmt = [self makeQuery:query];
    if (stmt == nil) {return; }

    NSMutableArray *array = [NSMutableArray array];
    while (sqlite3_step(stmt) == SQLITE_ROW) {
        int ident = sqlite3_column_int(stmt, 0);
        char * cname = (char*) sqlite3_column_text(stmt, 1);
        char * ccity = (char*) sqlite3_column_text(stmt, 2);
        char * cstate = (char*) sqlite3_column_text(stmt, 3);
        NSString * name = [NSString stringWithUTF8String:cname];
        NSString * city = [NSString stringWithUTF8String:ccity];
        NSString * state = [NSString stringWithUTF8String:cstate];        
        FailedBanksInfo* info = [[FailedBanksInfo alloc] initWithId:ident 
                                                              named:name 
                                                               city:city 
                                                              state:state];
        [array addObject:info];
    }
    sqlite3_finalize(stmt);
    
    _array = [NSArray arrayWithArray:array];
}

- (NSArray*) array
{
    if (_array) { return _array; }
    if (_sqliteDatabase == NULL) { return nil; }
    [self initArray];
    return _array;
}

- (NSString *) description
{
    NSMutableString * log = [NSMutableString string];
    NSArray * array = self.array;
    for (FailedBanksInfo* info in array) {
        [log appendFormat: @"%@\n", info];
    }
    return log;
}

- (sqlite3*) sqliteDatabase
{
    if (_sqliteDatabase == nil) {
        [self openSqliteDB];
    }
    return _sqliteDatabase;
}

- (void) openSqliteDB
{
    if(_sqliteDatabase != NULL) { return; }
    
    const char* db = self.sqlitePath;
    int status = sqlite3_open(db, &_sqliteDatabase);
    if (status != SQLITE_OK) {
        NSLog(@"Failed to open database");
    } else {
        NSLog(@"Suceeded opening database");
    }
}

- (void) closeSqliteDB
{
    if (_sqliteDatabase == NULL) { return; }
    
    sqlite3_close(_sqliteDatabase);
}

- (id) init
{
    self = [super init];
    if (self == nil) { return self; }
    [self openSqliteDB];
    return self;
}

- (void) dealloc
{
    [self closeSqliteDB];
//  [super dealloc];
}

#pragma mark factory functions

static FailedBanksDatabase * failedBankDatabaseSingleton = nil;

+ (FailedBanksDatabase*) failedBanksDatabase
{
    if (failedBankDatabaseSingleton != nil) { 
        return failedBankDatabaseSingleton;
    }
    failedBankDatabaseSingleton = [[FailedBanksDatabase alloc] init];
    return failedBankDatabaseSingleton;
}

+ (id) selectInfoAtIndexPath:(NSIndexPath*) path
{
    [FailedBanksDatabase failedBanksDatabase];
    NSInteger row = path.row;
    return [failedBankDatabaseSingleton.array objectAtIndex:row];
}

- (void) updateInfo:(FailedBanksInfo*) info
{
    NSString* ident = [NSString stringWithFormat:@"ident=%@", info.ident];
    NSString* bank = [NSString stringWithFormat:@"Bank_Name=\"%@\"", info.bankName];
    NSString* city = [NSString stringWithFormat:@"City=\"%@\"", info.city];
    NSString* state = [NSString stringWithFormat:@"State=\"%@\"", info.state];
    NSString *query = [NSString stringWithFormat:@"update banklist set %@,%@,%@ where %@", 
                bank, city, state, ident, nil];
    sqlite3_stmt* stmt = [self makeQuery:query];
    if (stmt == nil) {return; }
    int status = sqlite3_step(stmt);
    if (status != SQLITE_DONE) {
        // error
    }
    sqlite3_finalize(stmt);
    [self initArray];
}

- (void) insertInfo:(FailedBanksInfo*) info
{
    NSString *query = [NSString stringWithFormat:@"insert into banklist (Ident, Bank_Name, City, State) values(%@, \"%@\", \"%@\", \"%@\")", info.ident, info.bankName, info.city, info.state, nil];
    sqlite3_stmt* stmt = [self makeQuery:query];
    if (stmt == nil) {return; }
    int status = sqlite3_step(stmt);
    if (status != SQLITE_DONE) {
        // error
    }
    sqlite3_finalize(stmt);
    [self initArray];
}

- (void) reload
{
    [self initArray];
}

@end
