//
//  FailedBanksViewController.m
//  FailedBanks
//
//  Created by Timothy Shiu on 6/28/12.
//  Copyright (c) 2012 Shiu. All rights reserved.
//
#import <CoreData/CoreData.h>
#import "FailedBanksEditViewController.h"
#import "FailedBanksInfoViewController.h"
#import "FailedBanksViewController.h"
#import "FailedBanksCoreData.h"

@implementation FailedBanksViewController

@synthesize database = _database;
@synthesize coredata = _coredata;

- (void) reload
{
    [self.database reload];
    [self.tableView reloadData];
}

- (FailedBanksDatabase*) database
{
    if (_database != nil) { return _database; }
    _database = [FailedBanksDatabase failedBanksDatabase];
    return _database;
}

- (FailedBanksCoreDataEnv*) coredata
{
    if (_coredata != nil) { return _coredata; }
    _coredata = [[FailedBanksCoreDataEnv alloc] init];
    return _coredata;
}

- (NSInteger) infoNumberOfSections
{
    return 1; // for self.database.array
}

- (NSInteger) dataNumberofSections
{
    NSFetchedResultsController* fetcher = self.coredata.fetcher;
    return [[fetcher sections] count];    
}

- (NSInteger) infoNumberOfRowsInSection:(NSInteger) section
{
    return [self.database.array count];
}

- (NSInteger) dataNumberOfRowsInSection:(NSInteger) section
{
    NSFetchedResultsController* fetcher = self.coredata.fetcher;
    return [[[fetcher sections] objectAtIndex:section] numberOfObjects];
}

- (NSInteger) numberOfSectionsInTableView:(UITableView *)tableView
{
    // return [self dataNumberofSections];
    return [self infoNumberOfSections];
}

- (NSInteger) tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // return [self dataNumberOfRowsInSection:section];
    return [self infoNumberOfRowsInSection:section];
}

- (void) infoCell:(UITableViewCell*) cell atIndexPath:(NSIndexPath*) indexPath
{   
    NSArray* array = self.database.array;
    FailedBanksInfo *info = [array objectAtIndex:indexPath.row];
    NSString* subtitle = [NSString stringWithFormat:@"%@, %@", info.city, info.state];
    cell.detailTextLabel.text = subtitle;
    cell.textLabel.text = info.bankName;
}

- (void) dataCell:(UITableViewCell*) cell atIndexPath:(NSIndexPath*) indexPath
{
    FailedBanksCoreData* data = [self.coredata.fetcher objectAtIndexPath:indexPath];
    NSString* subtitle = [NSString stringWithFormat:@"%@, %@", data.city,
                          data.state];
    cell.detailTextLabel.text = subtitle;
    cell.textLabel.text = data.bankName;
}

- (UITableViewCell*) tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellID = @"BankSummaryTableCell";
    
    UITableViewCell *cell = nil;
    cell = [tableView dequeueReusableCellWithIdentifier:CellID];
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle 
                                       reuseIdentifier:CellID];
    }
    
    [self infoCell:cell atIndexPath:indexPath];
    // [self dataCell:cell atIndexPath:indexPath];
    return cell;  
}

- (void) setup
{
    self.navigationItem.title = @"Summary";    
}

- (void) awakeFromNib
{
    [self setup];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
	// Do any additional setup after loading the view, typically from a nib.
    [self setup];
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return YES;
}

#pragma mark segue

- (void) prepareForSegueOfSummary2Info:(UIStoryboardSegue *)segue sender:(id)sender
{
    NSLog(@"segue from summary to info");
    NSIndexPath* indexPath = [self.tableView indexPathForCell:sender];
    FailedBanksInfoViewController* target = segue.destinationViewController;
    // target.info  = [self.database.array objectAtIndex:indexPath.row];
    target.info = [FailedBanksDatabase selectInfoAtIndexPath:indexPath];
    // target.coredata = self.coredata;
    target.database = self.database;
    return;
    
    FailedBanksCoreData* data = [self.coredata.fetcher objectAtIndexPath:indexPath];
    FailedBanksInfo* info = [[FailedBanksInfo alloc] initWithId:[data.ident intValue] named:data.bankName city:data.city state:data.state];
    target.info = info;
    target.database = self.database;
    target.coredata = self.coredata;
}

- (void) prepareForSegueOfSummary2Edit:(UIStoryboardSegue *)segue sender:(id)sender
{
    NSLog(@"segue from summary to edit");
    FailedBanksEditViewController* target = nil;
    target = segue.destinationViewController;
    NSIndexPath* indexPath = [self.tableView indexPathForCell:sender];
    FailedBanksInfo* info = [[FailedBanksInfo alloc] init];
    target.info = info;
    [target.edit setTarget:target];
    [target.edit setTitle:@"insert"];
    [target.edit setAction:@selector(insertInfo:)];
    target.database = self.database;
    // target.coredata = self.coredata;
    // NSInteger ident = [self.database.array count] + 1;
    // target.info  = [self.database.array objectAtIndex:indexPath.row];    
    // target.info.ident = [NSNumber numberWithInteger:indent];
    // target.info.bankName = nil;
    // target.info.city = nil;
    // target.info.state = nil;
    
    return; 
        // FailedBanksInfo* info = [[FailedBanksInfo alloc] init];
    NSUInteger count = [[[self.coredata.fetcher sections] objectAtIndex:indexPath.section] numberOfObjects];
    info.ident = [NSNumber numberWithInt:count + 1];
    target.info = info;    
    target.fetcher = self.coredata.fetcher;
    target.context = self.coredata.context;
}

- (void) prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    NSArray* array = [NSArray arrayWithObjects:@"Summary2Info", @"Summary2Edit", nil];
    NSString* ident = segue.identifier;
    NSUInteger seg = [array indexOfObject:ident];
    switch (seg) {
        case 0: [self prepareForSegueOfSummary2Info:segue sender:sender]; break;
        case 1: [self prepareForSegueOfSummary2Edit:segue sender:sender]; break;
        default: break;
    }
}

@end
